"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.roundBuffer = exports.verifyBeacon = exports.verifySigOnG1 = void 0;
const bls = __importStar(require("@noble/bls12-381"));
const bls12_381_1 = require("@noble/bls12-381");
const index_1 = require("./index");
async function verifyBeacon(chainInfo, beacon) {
    const publicKey = chainInfo.public_key;
    if (!await randomnessIsValid(beacon)) {
        return false;
    }
    if ((0, index_1.isChainedBeacon)(beacon, chainInfo)) {
        return bls.verify(beacon.signature, await chainedBeaconMessage(beacon), publicKey);
    }
    if ((0, index_1.isUnchainedBeacon)(beacon, chainInfo)) {
        return bls.verify(beacon.signature, await unchainedBeaconMessage(beacon), publicKey);
    }
    if ((0, index_1.isG1G2SwappedBeacon)(beacon, chainInfo)) {
        return verifySigOnG1(beacon.signature, await unchainedBeaconMessage(beacon), publicKey);
    }
    console.error(`Beacon type ${chainInfo.schemeID} was not supported`);
    return false;
}
exports.verifyBeacon = verifyBeacon;
function normP1(point) {
    return point instanceof bls12_381_1.PointG1 ? point : bls12_381_1.PointG1.fromHex(point);
}
function normP2(point) {
    return point instanceof bls12_381_1.PointG2 ? point : bls12_381_1.PointG2.fromHex(point);
}
async function normP1Hash(point) {
    return point instanceof bls12_381_1.PointG1 ? point : bls12_381_1.PointG1.hashToCurve(point);
}
async function verifySigOnG1(signature, message, publicKey) {
    const P = normP2(publicKey);
    const Hm = await normP1Hash(message);
    const G = bls12_381_1.PointG2.BASE;
    const S = normP1(signature);
    const ePHm = (0, bls12_381_1.pairing)(Hm, P.negate(), false);
    const eGS = (0, bls12_381_1.pairing)(S, G, false);
    const exp = eGS.multiply(ePHm).finalExponentiate();
    return exp.equals(bls12_381_1.Fp12.ONE);
}
exports.verifySigOnG1 = verifySigOnG1;
async function chainedBeaconMessage(beacon) {
    const message = Buffer.concat([
        signatureBuffer(beacon.previous_signature),
        roundBuffer(beacon.round)
    ]);
    return bls.utils.sha256(message);
}
async function unchainedBeaconMessage(beacon) {
    return bls.utils.sha256(roundBuffer(beacon.round));
}
function signatureBuffer(sig) {
    return Buffer.from(sig, 'hex');
}
function roundBuffer(round) {
    const buffer = Buffer.alloc(8);
    buffer.writeBigUInt64BE(BigInt(round));
    return buffer;
}
exports.roundBuffer = roundBuffer;
async function randomnessIsValid(beacon) {
    const expectedRandomness = await bls.utils.sha256(Buffer.from(beacon.signature, 'hex'));
    return Buffer.from(beacon.randomness, 'hex').compare(expectedRandomness) == 0;
}
//# sourceMappingURL=beacon-verification.js.map